using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;

namespace MvcGallery3.Extensions.Html.Grid
{
    internal class GridColumnDescriptor
    {
        public String HeaderText { get; set; }
        public String BoundProperty { get; set; }
        public IDataTransformer Transformer { get; set; }


        internal static ICollection<GridColumnDescriptor> ExtractPropertiesFrom<T>()
        {
            var properties = typeof(T).GetProperties();
            var columns = properties.Select(p => new GridColumnDescriptor
                                                     {
                                                         HeaderText = GetDescription(p),
                                                         BoundProperty = p.Name,
                                                         Transformer = GetTransformer(p)
                                                     })
                                    .ToList();
            return columns;
        }

        internal static Object GetValueFor<T>(String propertyName, T instance, GridColumnDescriptor descriptor)
        {
            var theValue = typeof(T).GetProperty(propertyName).GetValue(instance, null);
            return descriptor.Transformer != null 
                        ? descriptor.Transformer.Apply(theValue) 
                        : theValue;
        }

        private static String GetDescription(PropertyInfo propertyInfo)
        {
            var attributes = propertyInfo.GetCustomAttributes(typeof(DescriptionAttribute), false);
            if (attributes.Length < 1)
                return propertyInfo.Name;

            var descriptionAttribute = (attributes[0] as DescriptionAttribute);
            return descriptionAttribute == null ? propertyInfo.Name : descriptionAttribute.Description;
        }

        private static IDataTransformer GetTransformer(PropertyInfo propertyInfo)
        {
            var attributes = propertyInfo.GetCustomAttributes(typeof(TransformerAttribute), false);
            if (attributes.Length < 1)
                return null;

            var transformerAttribute = (attributes[0] as TransformerAttribute);
            if (transformerAttribute == null)
                return null;

            return Activator.CreateInstance(transformerAttribute.Transformer) as IDataTransformer;
        }
    }
}