﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Mvc;
using System.Web.Routing;

namespace MvcGallery3.Extensions.Html
{
    public static class PagerHelper
    {
        public static MvcHtmlString Pager(this HtmlHelper helper, String name, Int32 count, Int32 pageSize, String baseUrl)
        {
            return Pager(helper, name, count, pageSize, baseUrl, 0, null);
        }
        public static MvcHtmlString Pager(this HtmlHelper helper, String name, Int32 count, Int32 pageSize, String baseUrl, Int32 pageIndex)
        {
            return Pager(helper, name, count, pageSize, baseUrl, pageIndex, null);
        }

        public static MvcHtmlString Pager(this HtmlHelper helper,
                                          String name,
                                          Int32 count,
                                          Int32 pageSize,
                                          String baseUrl,
                                          Int32 pageIndex,
                                          Object htmlAttributes)
        {
            // Convert from object to dictionary
            var dict = (IDictionary<String, Object>) new RouteValueDictionary(htmlAttributes);

            // Create a drop-down list with selectable pages
            var numOfPages = count / pageSize;
            if (count % pageSize > 0)
                numOfPages++;

            // Build the inner part of the pager bar
            var pagerRowBuilder = new StringBuilder("<tr>");
            for (var i = 1; i <= numOfPages; i++)
            {
                var formatNormal = "<a href='{0}?pageIndex={1}'>Page {1}</a>";
                var formatSelected = "<span>Page {0}</span>";
                var content = String.Empty;
                var cssClass = String.Empty;
                if (i == pageIndex)
                {
                    content = String.Format(formatSelected, i);
                    cssClass = "selectedPage";
                }
                else
                {
                    content = String.Format(formatNormal, baseUrl, i);
                }
                pagerRowBuilder.AppendFormat("<td class='{0}'>{1}</td>", cssClass, content);
            }
            pagerRowBuilder.Append("</tr>");

            // Build the pager bar
            var pager = new TagBuilder("table");
            pager.MergeAttributes(dict);
            pager.MergeAttribute("cellspacing", "0");
            pager.MergeAttribute("cellpadding", "2");
            pager.MergeAttribute("border", "0");
            pager.GenerateId(name);
            pager.InnerHtml = pagerRowBuilder.ToString();

            return MvcHtmlString.Create(pager.ToString());
        }
    }
}