﻿using System;
using System.IO;
using System.Threading;
using System.Web.Mvc;

namespace MvcGallery3.Extensions.Localization
{
    public static class UrlExtensions
    {
        public static String Content(this UrlHelper helper, String contentPath, Boolean localizable=false)
        {
            var url = contentPath;
            if (localizable)
            {
                url = GetLocalizedUrl(helper, url);
            }
            return helper.Content(url);
        }

        public static String GetLocalizedUrl(String resourceUrl)
        {
            var cultureExt = String.Format("{0}{1}",
                Thread.CurrentThread.CurrentUICulture.TwoLetterISOLanguageName, Path.GetExtension(resourceUrl));
            var url = Path.ChangeExtension(resourceUrl, cultureExt);
            return url;
        }

        public static String GetLocalizedUrl(UrlHelper helper, String resourceUrl)
        {
            var url = GetLocalizedUrl(resourceUrl);
            return VirtualFileExists(helper, url) ? url : resourceUrl;
        }

        public static Boolean VirtualFileExists(UrlHelper helper, String url)
        {
            var fullVirtualPath = helper.Content(url);
            var physicalPath = helper.RequestContext.HttpContext.Server.MapPath(fullVirtualPath);
            return File.Exists(physicalPath);
        }
    }
}