﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Threading;
using System.Web.Mvc;
using System.Xml;
using AsyncApp.Common;
using AsyncApp.Views;
using System.ServiceModel.Syndication;


namespace AsyncApp.Controllers
{
    public class HomeController : AsyncController
    {
        public ActionResult Index()
        {
            ViewData[ViewDataEntries.Welcome] = AppResources.WelcomeMessage;
            ViewData[ViewDataEntries.AppName] = GetType().Assembly.GetName().Name;
            return View();
        }

        public ActionResult Todo()
        {
            return View();
        }

        // Must be named xxxAsync (parameter subject to model binding, as usual)
        public void LookupAsync(SomeData data)
        {
            // Process input data
            //

            // Now you're ready to start the I/O bound operation
            //

            // Increment the counter of async operations
            AsyncManager.OutstandingOperations.Increment();

            // Start the I/O bound operation
            ThreadPool.QueueUserWorkItem(
                o =>
                {
                    // Do some work
                    Thread.Sleep(data.Id * 1000);

                    // Terminate operations
                    var d = new SomeResponse
                                {
                                    Data = String.Format("Waiting for {0} secs", data.Id),
                                    Message = "No message at this time"
                                };
                    AsyncManager.Parameters["data"] = d;
                    AsyncManager.OutstandingOperations.Decrement();
                },
                null);
        }

        public ActionResult LookupCompleted(SomeResponse data)
        {
            ViewData["d.FinalThread"] = String.Format("<b>Rendered by thread</b> {0}",
                        Thread.CurrentThread.ManagedThreadId);
            ViewData["d.Message"] = data.Message;
            ViewData["d.Data"] = data.Data;

            if (HttpContext.Request.IsAjaxRequest())
                return PartialView("LookupAjax");
            return View();
        }

        protected override void OnActionExecuting(ActionExecutingContext filterContext)
        {
            if (filterContext.ActionDescriptor.ActionName == "Lookup")
            {
                filterContext.Controller.ViewData["d.Started"] = String.Format(
                    "<b>Started</b> @ {0} on <b>thread</b> {1}<br>...<br>",
                    DateTime.Now, Thread.CurrentThread.ManagedThreadId);
            }

            base.OnActionExecuting(filterContext);
        }

        protected override void OnActionExecuted(ActionExecutedContext filterContext)
        {
            if (filterContext.ActionDescriptor.ActionName == "Lookup")
            {
                filterContext.Controller.ViewData["d.Finalized"] = String.Format(
                    "<b>Finalized</b> @ {0} on <b>thread</b> {1}<hr>",
                    DateTime.Now, Thread.CurrentThread.ManagedThreadId);
            }

            base.OnActionExecuted(filterContext);
        }




        //
        //
        // More significant example
        public void TennisNewsAsync()
        {
            var uri = new Uri("http://feeds2.feedburner.com/Tennis-AtpWorldTourHeadlineNews");
            AsyncManager.OutstandingOperations.Increment();

            var client = new WebClient();
            client.DownloadStringCompleted += (sender, e) =>
                                                  {
                                                      var feed = ProcessToFeed(e.Result);
                                                      AsyncManager.Parameters["feedItems"] = feed;
                                                      AsyncManager.OutstandingOperations.Decrement();
                                                  };
            client.DownloadStringAsync(uri);
        }

        public ActionResult TennisNewsCompleted(IList<SyndicationItem> feedItems)
        {
            return View(feedItems);
        }

        private static IList<SyndicationItem> ProcessToFeed(String xml)
        {
            var textReader = new StringReader(xml);
            var xmlReader = XmlReader.Create(textReader);
            var feed = SyndicationFeed.Load(xmlReader);
            return feed == null ? null : new List<SyndicationItem>(feed.Items);
        }
    }
}
