﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Web.Mvc;
using BasicInput.ViewModels.Complex;

namespace BasicInput.Controllers
{
    public class ComplexController : Controller
    {
        public ActionResult Repeat(RepeatText inputModel)
        {
            var model = new RepeatViewModel { Title = "Repeating text", Text = inputModel.Text, Number = inputModel.Number };
            return View(model);
        }

//        public ActionResult RepeatOnlyText([Bind(Exclude="number")]RepeatText inputModel)
        public ActionResult RepeatOnlyText(
            [Bind(Include = "text")]RepeatText inputModel)
        {
            // Number is excluded from binding (default to 1 as in ctor of RepeatText)
            var model = new RepeatViewModel { Title = "Repeating text", Text = inputModel.Text, Number = inputModel.Number };
            return View("repeat", model);
        }

        [ActionName("Emails")]
        [HttpGet]
        public ActionResult EmailForGet(IList<String> emails)
        {
            // Input parameters
            var defaultEmails = new[] { "admin@contoso.com", "", "", "", "" };
            if (emails == null)
                emails = defaultEmails;
            if (emails.Count == 0)
                emails = defaultEmails;

            var model = new EmailsViewModel {Emails = emails};
            return View(model);
        }

        [ActionName("Emails")]
        [HttpPost]
        public ActionResult EmailForPost(
            [Bind(Prefix = "email")]IList<String> emails)
        {
            var defaultEmails = new[] { "admin@contoso.com", "", "", "", "" };
            var model = new EmailsViewModel { Emails = defaultEmails, RegisteredEmails = emails };
            return View(model);
        }

        [ActionName("Countries")]
        public ActionResult ListCountriesForGet()
        {
            // Input parameters
            var defaultCountries = GetDefaultCountries();

            // Render the view
            var model = new ListCountriesViewModel { CountryList = defaultCountries };
            return View(model);
        }

        [HttpPost]
        [ActionName("Countries")]
        public ActionResult ListCountriesForPost(IList<Country> countries)
        {
            var defaultCountries = GetDefaultCountries();

            var model = new ListCountriesViewModel
                            {
                                CountryList = defaultCountries,
                                SelectedCountries = countries
                            };
            return View(model);
        }

        [HttpPost]
        [ActionName("RepeatForm")]
        public ActionResult RepeatForPost(RepeatInputModel inputModel)
        {
            var model = new RepeatViewModel { Title = "Repeating text", Number = inputModel.Number, Verb = "POST" };
            var text = inputModel.Text;
            if (inputModel.UseFormatting)
                text = String.Format("<b>{0}</b>", text);
            model.Text = text;

            return View("RepeatForPost", model);
        }

        [HttpGet]
        [ActionName("RepeatForm")]
        public ActionResult RepeatForGet()
        {
            var model = new RepeatViewModel { Title = "Repeating text", Number = 2, Text = "Dino", Verb = "GET" };
            return View("RepeatForGet", model);
        }

        private static IList<Country> GetDefaultCountries()
        {
            var countries = new List<Country>();
            countries.Add(new Country() { Name = "Italy", Details=new CountryInfo {Capital="Rome", Continent="Europe"}});
            countries.Add(new Country() { Name = "Spain", Details = new CountryInfo { Capital = "Madrid", Continent = "Europe" } });
            countries.Add(new Country() { Name = "USA", Details = new CountryInfo { Capital = "Washington", Continent = "NorthAmerica" } });
            return countries;
        }
    }
}
