﻿using System;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using MvcGallery3.Extensions.Binders;

namespace BasicInput
{
    public class MvcApplication : HttpApplication
    {
        /// <summary>
        /// Register application-specific global filters.
        /// </summary>
        /// <param name="filters">Built-in filter collection to extend</param>
        public static void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(new HandleErrorAttribute());
        }

        /// <summary>
        /// Register application-specific routes.
        /// </summary>
        /// <param name="routes">Built-in route collection to extend</param>
        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            routes.MapRoute(
                "Default", // Route name
                "{controller}/{action}/{id}", // URL with parameters
                new { controller = "Home", action = "Index", id = UrlParameter.Optional } // Parameter defaults
            );

            routes.MapRoute(
                "Test", // Route name
                "{controller}/{action}/test/{number}", // URL with parameters
                new { controller = "Binding", action = "RepeatWithPrecedence", number = 5 } // Parameter defaults
            );
        }

        /// <summary>
        /// Handler for the application startup point.
        /// </summary>
        protected void Application_Start()
        {
            RegisterGlobalFilters(GlobalFilters.Filters);
            RegisterRoutes(RouteTable.Routes);

            // Not invoked if another binder for the same type is defined on the method.
            ModelBinders.Binders[typeof(DateTime)] = new DateModelBinder();
        }
    }
}