﻿using System;
using System.Web.Mvc;
using Northwind.Model;
using Prg.WorkerServices.Abstractions;
using Prg.WorkerServices.Customers;

namespace Prg.Controllers
{
    public class CustomerController : Controller
    {
        private readonly ICustomerWorkerServices _service;

        public CustomerController() : this(new CustomerWorkerServices())
        {
        }
        public CustomerController(ICustomerWorkerServices service)
        {
            _service = service;
        }

        [HttpPost]
        [ActionName("edit")]
        public ActionResult EditViaPost([Bind(Prefix = "customerList")] String customerId)
        {
            return RedirectToAction("edit", new {id = customerId});
        }

        [HttpGet]
        [ActionName("edit")]
        public ActionResult EditViaGet(String id)
        {
            // Merge current ModelState with any being recovered from TempData
            LoadStateFromTempData();

            var model = _service.EditCustomer(id);
            return View("edit", model);
        }

        [HttpPost]
        public ActionResult Update(Customer customer)
        {
            _service.TryUpdateCustomer(ControllerContext, customer);
            return RedirectToAction("edit", new { id = customer.CustomerID });
        }

        private void LoadStateFromTempData()
        {
            var modelState = TempData["ModelState"] as ModelStateDictionary;
            if (modelState != null)
                ModelState.Merge(modelState);
        }
    }
}
