using System;
using System.Web.Mvc;
using Northwind.DAL.Repositories;
using Northwind.DAL.Repositories.Abstractions;
using Northwind.Model;
using Prg.ViewModels.Customers;
using Prg.WorkerServices.Abstractions;

namespace Prg.WorkerServices.Customers
{
    public class CustomerWorkerServices : ICustomerWorkerServices
    {
        private readonly ICustomerRepository _repository;

        public CustomerWorkerServices() : this(new CustomerRepository())
        {
        }
        public CustomerWorkerServices(ICustomerRepository repository)
        {
            _repository = repository;
        }

        public EditCustomerViewModel FindAllCustomers()
        {
            var model = new EditCustomerViewModel { Title = "Find all", Customers = _repository.FindAll() };
            return model;
        }

        public EditCustomerViewModel EditCustomer(String id)
        {
            var model = new EditCustomerViewModel
                            {
                                Title = "Edit customer",
                                Customer = _repository.FindById(id),
                                Customers = _repository.FindAll()
                            };
            return model;
        }

        public Boolean TryUpdateCustomer(ControllerContext context, Customer customer)
        {
            if (!Validate(context, customer))
                return false;

            if (!Update(context, customer))
                return false;

            return true;
        }




        private static Boolean Validate(ControllerContext context, Customer customer)
        {
            var result = true;
            if (String.IsNullOrEmpty(customer.Country) || !customer.Country.Equals("USA"))
            {
                var modelState = context.Controller.ViewData.ModelState;
                modelState.AddModelError("Country",
                    "Invalid country. Can only be 'USA'.");

                // Save model-state to TempData
                context.Controller.TempData["ModelState"] = modelState;
                result = false;
            }

            return result;
        }

        private Boolean Update(ControllerContext context, Customer customer)
        {
            // Perform physical update  
            var result = _repository.Update(customer);

            // Add a timeout message for the user
            var msg = result
                            ? "Successfully updated."
                            : "Update failed. Check your input data!";

            context.Controller.TempData["OutputMessage"] = msg;
            return result;
        }
    }
}