﻿using System;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using Caching.Common;

namespace Caching
{
    public class MvcApplication : HttpApplication
    {
        public static void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(new HandleErrorAttribute());
        }

        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            routes.MapRoute(
                "Default", // Route name
                "{controller}/{action}/{id}", // URL with parameters
                new { controller = "Home", action = "Index", id = UrlParameter.Optional } // Parameter defaults
            );

        }

        protected void Application_Start()
        {
            RegisterGlobalFilters(GlobalFilters.Filters);
            RegisterRoutes(RouteTable.Routes);

            // Inject a global caching service
            RegisterCacheService(new AspNetCacheService());

            // Store app-wide data
            CacheService["StartTime"] = DateTime.Now;
        }

        #region Cache injection
        private static ICacheService _internalCacheObject;
        public void RegisterCacheService(ICacheService cacheService)
        {
            _internalCacheObject = cacheService;
        }

        public static ICacheService CacheService
        {
            get { return _internalCacheObject;  }
        }
        #endregion
    }
}