using System;
using DotNetOpenAuth.OAuth.Messages;

namespace TwitterAuth.Framework
{
    public class AuthenticationOnlyTokenManager : ConfigTokenManagerBase
    {
        protected override void StoreAccessTokenCore(String consumerKey, String requestToken, String accessToken, String accessTokenSecret)
        {
            // Invoked past authorization, this method exchanges a request token (now authenticated) with an access token.
            TokensAndSecrets.Remove(requestToken);
            TokensAndSecrets[accessToken] = accessTokenSecret;
        }

        protected override String GetTokenSecretCore(String token)
        {
            // Invoked past authorization, this method gets the secret given a request or access token.
            return TokensAndSecrets[token];
        }

        protected override void StoreNewRequestTokenCore(UnauthorizedTokenRequest request, ITokenSecretContainingMessage response)
        {
            // Invoked before going to Twitter to store a newly generated request token for later use.
            TokensAndSecrets[response.Token] = response.TokenSecret;
        }
    }
}