///////////////////////////////////////////////////////////////////////
// Constants
var ViewTypes = {} || ViewTypes;
ViewTypes.Days = 1;
ViewTypes.Day = 2;
ViewTypes.Session = 3;
ViewTypes.Speaker = 4;
ViewTypes.Speakers = 5;
ViewTypes.Hidden = false;
ViewTypes.Display = true;


(function ($) {
    var _navigationBar = null,
        _contentPanel = null,
        _applicationBar = null,
        _panelDays = null,
        _panelDay = null,
        _panelSpeakers = null, 
        _panelSpeaker = null, 
        _panelDayId = null,
        _panelSessionId = null,
        _panelSpeakersId = null,
        _panelSpeakerId = null,
        _currentViewType = null,
        _methods = {}, 
        _registeredUpdaters = [];

    $.mainPanelController = function () {  
        _navigationBar = $("#navigationBarPanel");
        _contentPanel = $("#contentPanel");
        _appbar = $("#applicationBarPanel");
        _panelDays = $("#panelDays");

        _panelDay = $("#panelDay"); 
        _panelDayId = _panelDay.attr("id"); 
        _registeredUpdaters[_panelDayId] = UpdateHelpers.DayPanelUpdater;
        _panelSession = $("#panelSession");
        _panelSessionId = _panelSession.attr("id"); 
        _registeredUpdaters[_panelSessionId] = UpdateHelpers.SessionUpdater;
        _panelSpeakers = $("#panelSpeakers");
        _panelSpeakersId = _panelSpeakers.attr("id"); 
        _registeredUpdaters[_panelSpeakersId] = UpdateHelpers.SpeakerPanelUpdater;
        _panelSpeaker = $("#panelSpeaker");
        _panelSpeakerId = _panelSpeaker.attr("id"); 
        _registeredUpdaters[_panelSpeakerId] = UpdateHelpers.SpeakerUpdater;

        _methods = {
            setView : _setView,
            refreshView : _refreshView,
            back : _back
        };

        return _methods;
    };

    var _setView = function (viewType) {
        _currentViewType = viewType;
        _contentPanelSetView(viewType);
        if (viewType === ViewTypes.Days) {
            _navigationBarSetView(ViewTypes.Hidden);
            _applicationBarSetView(ViewTypes.Display);
            return this;
        };
        if (viewType !== ViewTypes.Days) {
            _navigationBarSetView(ViewTypes.Display);
            _applicationBarSetView(ViewTypes.Hidden);
            return this;
        };
        return this;
    };
    
    var _refreshView = function (viewType, data) {
        var func = null;
        if (viewType === ViewTypes.Day) {
            func = _registeredUpdaters[_panelDayId];
        };
        if (viewType === ViewTypes.Session) {
            func = _registeredUpdaters[_panelSessionId];
        };
        if (viewType === ViewTypes.Speakers) {
            func = _registeredUpdaters[_panelSpeakersId];
        };
        if (viewType === ViewTypes.Speaker) {
            func = _registeredUpdaters[_panelSpeakerId];
        };

        if (func != null)
            func(data);
        return this;
    };

    var _back = function() {
        var viewType = _getReturnViewType();
        _setView(viewType);
    };

    var _navigationBarSetView = function(shouldBeVisible) {
        if (shouldBeVisible) 
            _navigationBar.show();
        else
            _navigationBar.hide();
    };

    var _applicationBarSetView = function(shouldBeVisible) {
        if (shouldBeVisible) 
            _appbar.show();
        else
            _appbar.hide();
    };

    var _contentPanelSetView = function(viewType) {
        _hideAllChildPanels();
        _contentPanel.show();
        if (viewType === ViewTypes.Days) {
            _panelDays.show();
            return;
        }
        if (viewType === ViewTypes.Day) {
            _panelDay.show();
            return;
        }    
        if (viewType === ViewTypes.Session) {
            _panelSession.show();
            return;
        }  
        if (viewType === ViewTypes.Speakers) {
            _panelSpeakers.show();
            return;
        }  
        if (viewType === ViewTypes.Speaker) {
            _panelSpeaker.show();
            return;
        }  
    };

    var _hideAllChildPanels = function() {
        _panelDays.hide();
        _panelDay.hide();
        _panelSession.hide();
        _panelSpeaker.hide();
        _panelSpeakers.hide();
    };

    var _getReturnViewType = function() {
        if (_currentViewType === ViewTypes.Days) {
            return _currentViewType;
        }
        if (_currentViewType === ViewTypes.Day) {
            return ViewTypes.Days;
        }    
        if (_currentViewType === ViewTypes.Session) {
            return ViewTypes.Day;        
        }  
        if (_currentViewType === ViewTypes.Speakers) {
            return ViewTypes.Days;
        }  
        if (_currentViewType === ViewTypes.Speaker) {
            return ViewTypes.Speakers;
        }  
    };
})(jQuery);